import { watchEffect, unref, getCurrentInstance } from 'vue-demi';
import { getTargetElement } from '../libs/dom';
import { isServer } from '../libs/helper';
const defaultEvent = 'click';
/**
 * useClickAway function
 *
 * @param target Execution handler target.
 * @param eventHandler Handler function on external click.
 * @param eventName Event trigger.
 */
function useClickAway(target, eventHandler, eventName = defaultEvent) {
  if (!getCurrentInstance()) {
    throw new Error(
      'Invalid hook call: `useClickAway` can only be called inside of `setup()`.',
    );
  }
  const handler = event => {
    const targets = Array.isArray(target) ? target : [target];
    if (
      targets.some(targetItem => {
        const targetElement = getTargetElement(targetItem);
        return !targetElement || targetElement.contains(event.target);
      })
    ) {
      return;
    }
    unref(eventHandler)(event);
  };
  watchEffect(onInvalidate => {
    if (!isServer) {
      const eventNames = Array.isArray(unref(eventName))
        ? unref(eventName)
        : [eventName];
      eventNames.map(name => document.addEventListener(unref(name), handler));
      onInvalidate(() => {
        eventNames.map(name =>
          document.removeEventListener(unref(name), handler),
        );
      });
    }
  });
}
export default useClickAway;
