import {
  ref,
  unref,
  readonly,
  watchEffect,
  getCurrentInstance,
} from 'vue-demi';
import { isFunction } from '../libs/helper';
// TODO: COMMENT NEED
export default function useInterval(
  callback,
  interval = 1000,
  immediate = true,
) {
  if (!getCurrentInstance()) {
    throw new Error(
      'Invalid hook call: `useInterval` can only be called inside of `setup()`.',
    );
  }
  let timer = null;
  const isActive = ref(immediate);
  const stop = () => {
    isActive.value = false;
    if (timer) {
      clearInterval(timer);
      timer = null;
    }
  };
  const start = () => {
    stop();
    isActive.value = true;
    timer = setInterval(() => {
      isFunction(callback) && callback();
    }, unref(interval));
  };
  watchEffect(onInvalidate => {
    unref(immediate) && start();
    onInvalidate(stop);
  });
  return {
    isActive: readonly(isActive),
    start,
    stop,
  };
}
