// https://github.com/antfu/vueuse/blob/master/packages/core/useStorage/index.ts
import {
  isDef,
  isUndef,
  isUndefined,
  isObject,
  isArray,
} from '../../libs/helper';
const getType = value =>
  value === null
    ? 'null'
    : typeof value === 'boolean'
    ? 'boolean'
    : typeof value === 'string' || isUndefined(value)
    ? 'string'
    : isObject(value) || isArray(value)
    ? 'object'
    : !Number.isNaN(value)
    ? 'number'
    : 'any';
export const Serializers = {
  any: {
    read: (v, d) => (v !== null && v !== void 0 ? v : d),
    write: v => String(v),
  },
  boolean: {
    read: (v, d) => (isDef(v) ? v === 'true' : d),
    write: v => String(v),
  },
  number: {
    read: (v, d) => (isDef(v) ? Number.parseFloat(v) : d),
    write: v => String(v),
  },
  object: {
    read: (v, d) => (v ? JSON.parse(v) : d),
    write: v => JSON.stringify(v),
  },
  string: {
    read: (v, d) => (isDef(v) ? v : d),
    write: v => String(v),
  },
  null: {
    read: () => null,
    write: () => null,
  },
};
export const read = (value, defaultValue) => {
  let type;
  const $value = isUndef(value) ? defaultValue : value;
  try {
    type = getType(JSON.parse($value));
  } catch (err) {
    type = getType($value);
  }
  return Serializers[type].read(value, defaultValue);
};
export const write = value => Serializers[getType(value)].write(value);
