import {
  ref,
  unref,
  readonly,
  watchEffect,
  getCurrentInstance,
} from 'vue-demi';
import { isFunction } from '../libs/helper';
// TODO: COMMENT NEED
export default function useTimeout(callback, timeout = 1000, immediate = true) {
  if (!getCurrentInstance()) {
    throw new Error(
      'Invalid hook call: `useTimeout` can only be called inside of `setup()`.',
    );
  }
  let timer = null;
  const isActive = ref(immediate);
  const stop = () => {
    isActive.value = false;
    if (timer) {
      clearTimeout(timer);
      timer = null;
    }
  };
  const start = () => {
    stop();
    isActive.value = true;
    timer = setTimeout(() => {
      isFunction(callback) && callback();
      isActive.value = false;
    }, unref(timeout));
  };
  watchEffect(onInvalidate => {
    unref(immediate) && start();
    onInvalidate(stop);
  });
  return {
    isActive: readonly(isActive),
    start,
    stop,
  };
}
