import { ref, unref, computed } from 'vue-demi';
import { isDef } from '../libs/helper';
/**
 * useToggle function
 *
 * @param defaultValue Truth value, default is `true`  - an optional parameter
 * @param reverseValue False value, default is `false` - an optional parameter
 * @returns [ status, { toggle, setLeft, setRight } ]
 */
function useToggle(defaultValue, reverseValue) {
  const getDefault = () =>
    isDef(unref(defaultValue)) ? unref(defaultValue) : true;
  const getReverse = () =>
    isDef(unref(reverseValue)) ? unref(reverseValue) : !getDefault();
  const state = ref(getDefault());
  const toggle = value => {
    state.value = isDef(unref(value))
      ? unref(value)
      : state.value !== getDefault()
      ? getDefault()
      : getReverse();
  };
  const setLeft = () => {
    state.value = getDefault();
  };
  const setRight = () => {
    state.value = getReverse();
  };
  return {
    state: computed({
      get: () => state.value,
      set: value => toggle(value),
    }),
    setLeft,
    setRight,
    toggle,
  };
}
export default useToggle;
