/* istanbul ignore file */
// Based on Fullscreen API.
// https://developer.mozilla.org/en-US/docs/Web/API/Fullscreen_API
import { isFunction, isServer } from './helper';
import { isHTMLElement } from './dom';
const eventsMap = {
  change: 'fullscreenchange',
  error: 'fullscreenerror',
};
const getFullscreenElement = () =>
  isServer ? null : document.fullscreenElement;
export const request = (element = document.body) => {
  if (!isServer && document.fullscreenEnabled) {
    if (isHTMLElement(element) && element.requestFullscreen) {
      element.requestFullscreen();
    } else {
      throw new Error(
        `Invalid assignment: expected a DOM Element but got: ${typeof element}`,
      );
    }
  } else {
    throw new Error(
      'Invalid call: cannot request fullscreen right now because fullscreen is disabled.',
    );
  }
};
export const exit = (element = document.body) => {
  if (
    !isServer &&
    isFunction(document.exitFullscreen) &&
    getFullscreenElement() === element
  ) {
    document.exitFullscreen();
  }
};
export const toggle = (element = document.body) => {
  !isServer && getFullscreenElement() === element ? exit() : request(element);
};
export const on = (event, fn) => {
  const eventName = eventsMap[event];
  if (eventName && !isServer) {
    document.addEventListener(eventName, fn, false);
  }
};
export const off = (event, fn) => {
  const eventName = eventsMap[event];
  if (eventName && !isServer) {
    document.removeEventListener(eventName, fn, false);
  }
};
export default {
  getFullscreenElement,
  request,
  exit,
  toggle,
  on,
  off,
};
